package tests;

import org.junit.*;
import static org.junit.Assert.*;

import javax.tools.*;
import java.util.*;

import com.sun.source.tree.*;

/**
 * JUnit tests for the Interned annotation checker.
 */
public class InternedTest extends CheckerTest {

    public InternedTest() {
        checkerName = "checkers.interned.InternedChecker";
        checkerDir = "interned";
    }

    /** Tests arrays. */
//    @Test public void testArrays() {
//        TestRun run = getTest("Arrays.java");
//        assertSuccess(run);
//        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
//        assertDiagnostics(list, "Arrays.out");
//    }

    /** Tests the basic semantics of Interned. */
    @Test public void testComparison() {
        TestRun run = getTest("Comparison.java");
        assertFailure(run);
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "Comparison.out");
    }

    /** Tests Interned checks in complex conditional expressions. */
    @Test public void testComplexComparison() {
        TestRun run = getTest("ComplexComparison.java");
        assertFailure(run);
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "ComplexComparison.out");
    }

    /** Tests that Interned works properly with object creation expressions. */
    @Test public void testCreation() {
        TestRun run = getTest("Creation.java");
        assertFailure(run);
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "Creation.out");
    }

    /** Tests that Interned works properly with enumerated types. */
    @Test public void testEnumerations() {
        TestRun run = getTest("Enumerations.java");
        assertSuccess(run);
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "Enumerations.out");
    }

    /** Tests that Interned works properly with expressions. */
//    @Test public void testExpressions() {
//        TestRun run = getTest("Expressions.java");
//        assertSuccess(run);
//        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
//        assertDiagnostics(list, "Expressions.out");
//    }

    /** Tests that Interned works properly with generic types. */
//    @Test public void testGenerics() {
//        TestRun run = getTest("Generics.java");
//        assertFailure(run);
//        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
//        assertDiagnostics(list, "Generics.out");
//    }

    /** Tests a standard usage scenario. */
    @Test public void testInternMethod() {
        TestRun run = getTest("InternMethod.java");
        assertSuccess(run);
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
       assertDiagnostics(list, "InternMethod.out");
    }

    /** Tests a standard usage scenario. */
//    @Test public void testInternedClass() {
//        TestRun run = getTest("InternedClass.java");
//        assertSuccess(run);
//        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
//        assertDiagnostics(list, "InternedClass.out");
//    }

    /** Tests that primitive types are implicitly treated as Interned. */
    @Test public void testMissingThisAnnotated() {
        TestRun run = getTest("MissingThisAnnotated.java");
        assertFailure(run);
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "MissingThisAnnotated.out");
    }

    /** Tests that primitive types are implicitly treated as Interned. */
    @Test public void testPrimitives() {
        TestRun run = getTest("Primitives.java");
        assertSuccess(run);
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "Primitives.out");
    }

    /** Tests another standard usage scenario. */
    @Test public void testStaticInternMethod() {
        TestRun run = getTest("StaticInternMethod.java");
        assertSuccess(run);
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "StaticInternMethod.out");
    }

    /** Tests that the return type of String.intern is implicitly Interned. */
    @Test public void testStringIntern() {
        TestRun run = getTest("StringIntern.java");
        assertFailure(run);
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "StringIntern.out");
    }

    /** Tests that SuppressWarnings works on variable declarations. */
    @Test public void testSuppressWarningsVar() {
        TestRun run = getTest("SuppressWarningsVar.java");
        assertSuccess(run);
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "SuppressWarningsVar.out");
    }

}
