package tests;

import org.junit.*;
import static org.junit.Assert.*;

import javax.tools.*;
import java.util.*;

import com.sun.source.tree.*;

/**
 * JUnit tests for the Javari annotation checker.
 */
public class JavariTest extends CheckerTest {

    public JavariTest() {
        checkerName = "checkers.javari.JavariChecker";
        checkerDir = "javari";
    }

    /** Tests for allowed initializers */
    @Test public void testLegalInitializers() {
        TestRun run = getTest("Initializers.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "Initializers.out");
    }

    /** Tests for illegal initializers */
    @Test public void testIllegalInitializers() {
        TestRun run = getTest("Initializers2.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "Initializers2.out");
    }

    /** Tests for legal assignments */
    @Test public void testLegalAssignments() {
        TestRun run = getTest("Assignments.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "Assignments.out");
    }

    /** Tests for illegal assignments */
    @Test public void testIllegalAssignments() {
        TestRun run = getTest("Assignments2.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "Assignments2.out");
    }

    /** Tests for null mutability */
    @Test public void testNull() {
        TestRun run = getTest("NullTester.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "NullTester.out");
    }

    /** Tests for constructors */
    @Test public void testConstructors() {
        TestRun run = getTest("Constructors.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "Constructors.out");
    }

    /** Tests for RoMaybe local variables */
    @Test public void testRoMaybeVariables() {
        TestRun run = getTest("RoMaybes.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "RoMaybes.out");
    }

    /** Tests for extensions */
    @Test public void testExtensions() {
        TestRun run = getTest("Extensions.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "Extensions.out");
    }

    /** Tests for array access */
    @Test public void testArrayAccess() {
        TestRun run = getTest("ArrayTest.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "ArrayTest.out");
    }

    /** Tests for conditional expressions */
    @Test public void testConditionalExpressions() {
        TestRun run = getTest("ConditionalExpressionTest.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "ConditionalExpressionTest.out");
    }

    /** Tests for for enhanced loops */
    @Test public void testForEnhanced() {
        TestRun run = getTest("ForEnhanced.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "ForEnhanced.out");
    }

    /** Tests for readonly classes */
    @Test public void testReadOnlyClass() {
        TestRun run = getTest("RoClass.java");
        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
        assertDiagnostics(list, "RoClass.out");
    }
}
